// vim: foldmethod=marker

//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

flock.xul.TreeClicker = function (aTree, aOpts) {/*{{{*/
    flock.xul.ClickSplitter.call(this, aTree, aOpts);
};/*}}}*/
dojo.inherits(flock.xul.TreeClicker, flock.xul.ClickSplitter);
dojo.lang.extend(flock.xul.TreeClicker, {
    handle: function (aEvent) {/*{{{*/
        var row = {};
        var col = {};
        var obj = {};
        
        // Get the call values
        this.mNode.treeBoxObject
            .getCellAt(aEvent.clientX, aEvent.clientY, row, col, obj);
        
        // Filter out invalid rows
        if (row.value == -1) {
            return;
        }

        // Filter out invalid columns
        if (!col.value) {
            return;
        }
        // Send to a handler
        if (obj.value == "twisty") {
            this.performAction("onTwisty", row.value, col.value);
        }
        else {
            var mods = this.split(aEvent);
            var method = "on" + mods.join('') + "Click";
            this.performAction(method, row.value, col.value);
        }
    },/*}}}*/
    performAction: function (aAction, aRow, aCol) {/*{{{*/
        this.mNode.view.performActionOnCell(aAction, aRow, aCol);
        this.mNode.view.performActionOnRow(aAction, aRow);
        this.mNode.view.performAction(aAction);
    },/*}}}*/
});


// options --
//  filterChildren - bool - do not add children of parents that are also added
//
function flock_getTreeSelection(aTree, aCols, aOpts) {/*{{{*/
    XXX('DEPRECATED', 'please use the getSelected method on a flock.xul.Tree instance instead');
    if (!aOpts) aOpts = {};
    var indexList = [];
    
    var rangeMin = {};
    var rangeMax = {};
    aTree.view.selection.getRangeAt(0, rangeMin, rangeMax);
    var out = [];
    var rangeCount = aTree.view.selection.getRangeCount();
    XXX('RANGE INIT', rangeMin.value, rangeMax.value);
    XXX('RANGECOUNT', rangeCount);
    for (var j = 0; j < rangeCount; ++j) {
        var dummy = {};
        aTree.view.selection.getRangeAt(j, dummy, rangeMax);
        XXX('RANGE', rangeMin.value, rangeMax.value);
    }
    XXX('RANGE END', rangeMin.value, rangeMax.value);
    
    for (var i = rangeMin.value; i <= rangeMax.value; ++i) {
        if (!aTree.view.selection.isSelected(i)) continue;
        var rowIndex = i;
        var parentIndex = aTree.view.getParentIndex(i);
        var isContainer = aTree.view.isContainer(i);
        if (aOpts['filterChildren']
            && indexList.indexOf(parentIndex) != -1) {
            continue;
        }
        indexList.push(rowIndex);
        var row = {
            rowIndex: rowIndex, 
            parentIndex: parentIndex, 
            isContainer: isContainer
        };
        
        row["parentIndex"] = aTree.view.getParentIndex(i);
        row["isContainer"] = aTree.view.isContainer(i);
        for (var j = 0; j < aCols.length; ++j) {
            var colName = aCols[j];
            var col = aTree.columns.getNamedColumn(colName);
            row[colName] = aTree.view.getCellText(i, col);
        }
        out.push(row);
    }
    return out;
}/*}}}*/

function flock_getTreeSelectionFromDrag(aEvt, aTree, aCol, aOpts) {
    if(aEvt.target.nodeName!="treechildren") return null;
    return flock_getTreeSelection(aTree, aCol, aOpts);
}

flock.xul.Tree = function (aTree, aOpts) {/*{{{*/
    flock.xul.Node.call(this, aTree, aOpts);
};/*}}}*/
dojo.inherits(flock.xul.Tree, flock.xul.Node);
dojo.lang.extend(flock.xul.Tree, {
    isDebug: false,
    isSorted: function () {/*{{{*/
        return this.mNode.view.isSorted();
    },/*}}}*/
    get mColumnsNode() {/*{{{*/
        for (var i = 0; i < this.mNode.childNodes.length; ++i) {
            var item = this.mNode.childNodes[i];
            if (item.tagName == "treecols") return item;
        }
        return null;
    },/*}}}*/
    get columns() {/*{{{*/
        var cols = [];
        for (var i = 0; i < this.mColumnsNode.childNodes.length; ++i) {
            var item = this.mColumnsNode.childNodes[i];
            if (item.tagName == "treecol") {
                cols.push(item);
            }
        }
        return cols;
    },/*}}}*/
    getIndexInParent: function (aIndex) {/*{{{*/
        var pIndex = this.mNode.view.getParentIndex(aIndex);
        var sIndex = pIndex + 1;
        var cIndex = sIndex;
        var i = 0;
        while (cIndex < aIndex) {
            // if they have the same parent...
            if (this.mNode.view.getParentIndex(cIndex) == pIndex) {
                i++;
            }
            cIndex++;
        }
        return i;
    },/*}}}*/
});

flock.xul.RDFTree = function (aTree, aOpts) {/*{{{*/
    flock.xul.Tree.call(this, aTree, aOpts);
}/*}}}*/
dojo.inherits(flock.xul.RDFTree, flock.xul.Tree);
dojo.lang.extend(flock.xul.RDFTree, {
    isSorted: function () {/*{{{*/
        if (!flock.xul.Tree.prototype.isSorted.call(this)) return false;
        for (var i = 0; i < this.columns.length; ++i) {
            var col = this.columns[i];
            if (col.getAttribute('sortActive') == 'true') {
                if (col.getAttribute('sortDirection') == 'natural') {
                    return false;
                }
            }
        }
        return true;
    },/*}}}*/
    
    // nsIXULRDFTree
    // Constants/*{{{*/
    DROP_BEFORE: -1,
    DROP_ON: 0,
    DROP_AFTER: 1,/*}}}*/
    canDrop: function (aIndex, aOrientation) {/*{{{*/
        if (this.isDebug) {
            return true;
        }
    },/*}}}*/
    onCycleCell: function (aRow, aCol) {/*{{{*/
    },/*}}}*/
    onCycleHeader: function (aRow, aElement) {/*{{{*/
    },/*}}}*/
    onDrop: function (aRow, aOrientation) {/*{{{*/
    },/*}}}*/
    onPerformAction: function (aAction) {/*{{{*/
        this.mNode.focus();
        document.commandDispatcher.updateCommands("focus");
    },/*}}}*/
    onPerformActionOnCell: function (aAction, aRow, aCol) {/*{{{*/
    },/*}}}*/
    onPerformActionOnRow: function (aAction, aRow) {/*{{{*/
    },/*}}}*/
    onSelectionChanged: function () {/*{{{*/
        this.mNode.focus();
        document.commandDispatcher.updateCommands("focus");
    },/*}}}*/
    onToggleOpenState: function (aIndex) {/*{{{*/
    },/*}}}*/

    getResourceAtIndex: function (aIndex) {/*{{{*/
        return this.mNode.builder.getResourceAtIndex(aIndex);
    },/*}}}*/
    getIndexOfResource: function (aRsrc) {/*{{{*/
        return this.mNode.builder.getIndexOfResource(aRsrc);
    },/*}}}*/
    getParentIndex: function (aRow) {/*{{{*/
        return this.mNode.view.getParentIndex(aRow);
    },/*}}}*/

    // the cool stuff
    getParentResource: function (aIndex) {/*{{{*/
        try {
            var pIndex = this.mNode.view.getParentIndex(aIndex);
        }
        catch (e) {
            var pIndex = -1;
        }
        if (pIndex == -1) return flock_getRDFResource(this.mNode.ref);
        return this.mNode.builder.getResourceAtIndex(pIndex);
    },/*}}}*/
    getSelected: function (aCols, aOpts) {/*{{{*/
        if (!aOpts) aOpts = {};
        var indexList = [];
        
        // XXX andy: this stuff is kind of annoying, is there a better way?
        var rangeMin = {};
        var rangeMax = {};
        this.mNode.view.selection.getRangeAt(0, rangeMin, rangeMax);
        var out = [];
        var rangeCount = this.mNode.view.selection.getRangeCount();
        XXX('RANGE INIT', rangeMin.value, rangeMax.value);
        XXX('RANGECOUNT', rangeCount);
        for (var j = 0; j < rangeCount; ++j) {
            var dummy = {};
            this.mNode.view.selection.getRangeAt(j, dummy, rangeMax);
            XXX('RANGE', rangeMin.value, rangeMax.value);
        }
        XXX('RANGE END', rangeMin.value, rangeMax.value);
        
        for (var i = rangeMin.value; i <= rangeMax.value; ++i) {
            if (!this.mNode.view.selection.isSelected(i)) continue;

            var rowIndex = i;
            var parentIndex = this.mNode.view.getParentIndex(i);
            var isContainer = this.mNode.view.isContainer(i);
            
            // filtering children, if we want to
            if (aOpts['filterChildren']
                && indexList.indexOf(parentIndex) != -1) {
                continue;
            }
            indexList.push(rowIndex);
            
            var row = {
                rowIndex: rowIndex, 
                parentIndex: parentIndex, 
                isContainer: isContainer
            };
            
            // if we specified columns, grab those, else get all
            if (aCols) {
                for (var j = 0; j < aCols.length; ++j) {
                    var colName = aCols[j];
                    var col = this.mNode.columns.getNamedColumn(colName);
                    row[colName] = this.mNode.view.getCellText(i, col);
                }
            }
            else {
                for (var j = 0; j < this.mNode.columns.count; ++j) {
                    var col = this.mNode.columns.getColumnAt(j);
                    row[col.id] = this.mNode.view.getCellText(i, col);
                }
            }
            out.push(row);
        }
        return out;        
    },/*}}}*/
    
    attach: function () {/*{{{*/
        if (this._isAttached) return;
        flock.xul.Node.prototype.attach.apply(this);
        this.mNode.builder.addObserver(this);
        // XXX andy: seems like onSelectionChanged does not actually
        //           get called unless we attach it explicitly.
        dojo.event.connect(this.mNode, 'onselect', 
                           this, 'onSelectionChanged');
        this._isAttached = true;
    },/*}}}*/
    detach: function () {/*{{{*/
        if (!this._isAttached) return;
        flock.xul.Node.prototype.detach.apply(this);
        this.mNode.builder.removeObserver(this);
        dojo.event.disconnect(this.mNode, 'onselect', 
                              this, 'onSelectionChanged');
        this._isAttached = false;
    },/*}}}*/
});






var flock_treeBuilderObserver = function () {};/*{{{*/
flock_treeBuilderObserver.prototype = {
    DROP_BEFORE: -1,
    DROP_ON: 0,
    DROP_AFTER: 1, 

    /**
     * Methods used by the drag feedback code to determine if a drag is allowable at
     * the current location. To get the behavior where drops are only allowed on
     * items, such as the mailNews folder pane, always return false whe
     * the orientation is not DROP_ON.
     */
    // boolean canDrop(in long index, in long orientation);
    canDrop: function (aIndex, aOrientation) { },
    
    /**
     * Called when the user drops something on this view. The |orientation| param
     * specifies before/on/after the given |row|.
     */
    //void onDrop(in long row, in long orientation);
    onDrop: function (aRow, aOrientation) { },
 
    /** 
     * Called when an item is opened or closed. 
     */
    //void onToggleOpenState (in long index);
    onToggleOpenState: function (aIndex) { },

    /** 
	 * Called when a header is clicked.
     */
    //void onCycleHeader(in wstring colID, in nsIDOMElement elt);
    onCycleHeader: function (aCol, aElement) { },

    /**
     * Called when a cell in a non-selectable cycling column (e.g. 
     * unread/flag/etc.) is clicked.
     */
    //void onCycleCell(in long row, in wstring colID);
    onCycleCell: function (aRow, aCol) { },

    /** 
     * Called when selection in the tree changes
     */
    //void onSelectionChanged();
    onSelectionChanged: function () { },

    /**
     * A command API that can be used to invoke commands on the selection.  
     * The tree will automatically invoke this method when certain keys 
     * are pressed.  For example, when the DEL key is pressed, performAction 
     * will be called with the "delete" string. 
     */
    //void onPerformAction(in wstring action);
    onPerformAction: function (aAction) { },

    /**
     * A command API that can be used to invoke commands on a specific row.
     */
    //void onPerformActionOnRow(in wstring action, in long row);
    onPerformActionOnRow: function (aAction, aRow) { },

    /**
     * A command API that can be used to invoke commands on a specific cell.
     */
    //void onPerformActionOnCell(in wstring action, in long row, in wstring colID);    
    onPerformActionOnCell: function (aAction, aRow, aCol) { }
    
}/*}}}*/

// DEPRECATED: you should probably not be using this function
function flock_treeMouseEvent(aEvent) {/*{{{*/
    XXX('DEPRECATED','flock_treeMouseEvent',
        'you should probably not be using this function, you probably want to use a higher performance tree');
    var row = {};
    var col = {};
    var obj = {};
    
    var tree
    //if (aEvent.target.nodeName == "tree") {
    //    tree = aEvent.target;
    //} else if (aEvent.target.parentNode.nodeName == "tree") {
    if (aEvent.target.parentNode.nodeName == "tree") {
        tree = aEvent.target.parentNode;
    } else {
        return;
    }
    //XXX("TARGET",aEvent.target.nodeName);
    //XXX("TREE",tree.nodeName);
    var treeBox = tree.treeBoxObject;
    //dd(tree);
    // Get the call values
    treeBox.getCellAt(aEvent.clientX, aEvent.clientY, row, col, obj);
    
    if (row.value == -1) {
        //XXX("INVALID ROW");
        return;
    }

    if (!col.value) {
        return;
    }

    // Punt to the regular handlers for twisties
    if (obj.value == "twisty" && aEvent.type == "click") {
        return;
    }
    // ... and dblclicks
    if (aEvent.type == "dblclick") {
        tree.view.toggleOpenState(row.value);
        return;
    }
    
    

    var treeCell = flock_treeGetCell(tree, row.value, col.value.index);
    if (!treeCell) {
        return;
    }
    
    var newEvent = document.createEvent("MouseEvent");
    newEvent.initMouseEvent(
        aEvent.type,
        aEvent.canBubble,
        aEvent.cancelable,
        aEvent.view,
        aEvent.detail,
        aEvent.screenX,
        aEvent.screenY,
        aEvent.clientX,
        aEvent.clientY,
        aEvent.ctrlKey,
        aEvent.altKey,
        aEvent.shiftKey,
        aEvent.metaKey,
        aEvent.button,
        treeCell
    );

    treeCell.dispatchEvent(newEvent);
    
}/*}}}*/
function flock_treeGetCell(aTree, aRowIndex, aColIndex) {/*{{{*/
    var treeRow = flock_treeGetRow(aTree, aRowIndex);
    if (!treeRow) return;

    if (treeRow.childNodes.length <= aColIndex) {
        return;
    }

    var treeCell = treeRow.childNodes[aColIndex];
    return treeCell;
}/*}}}*/
function flock_treeGetRow(aTree, aRowIndex) {/*{{{*/
    var treeItem = aTree.view.getItemAtIndex(aRowIndex);
    
    if (!treeItem.hasChildNodes()) {
        return;
    }
    var treeRow = treeItem.firstChild;
    return treeRow;
}/*}}}*/
